"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.clone = void 0;
const utils_1 = require("../utils");
const index_1 = require("./index");
const clone = _val => {
    const set = new WeakSet();
    function internalClone(val) {
        let ret;
        if (!(0, utils_1.isObject)(val)) {
            return val;
        }
        if (val === null) {
            return null; // null has typeof "object"
        }
        if (val instanceof Date) {
            return new Date(val.getTime());
        }
        // RegExps are not really EJSON elements (eg we don't define a serialization
        // for them), but they're immutable anyway, so we can support them in clone.
        if (val instanceof RegExp) {
            return val;
        }
        if (val.constructor.name === 'ObjectId' && (0, utils_1.isFunction)(val.toString)) {
            return val.toString();
        }
        if (val.constructor.name === 'model' && (0, utils_1.isObject)(val._doc)) {
            return internalClone(val._doc);
        }
        if (index_1.EJSON.isBinary(val)) {
            ret = index_1.EJSON.newBinary(val.length);
            for (let i = 0; i < val.length; i++) {
                ret[i] = val[i];
            }
            return ret;
        }
        function cloneArray(arr) {
            set.add(arr);
            return arr
                .map(val => {
                if ((0, utils_1.isObjectAndNotNull)(val)) {
                    if (set.has(val)) {
                        return undefined;
                    }
                    set.add(val);
                }
                return (0, exports.clone)(val);
            })
                .filter(val => val !== undefined);
        }
        if (Array.isArray(val)) {
            return cloneArray(val);
        }
        if ((0, utils_1.isArguments)(val)) {
            set.add(val);
            return cloneArray(Array.from(val));
        }
        // handle general user-defined typed Objects if they have a clone method
        if ((0, utils_1.isFunction)(val.clone)) {
            return val.clone();
        }
        // handle other custom types
        if (index_1.EJSON._isCustomType(val)) {
            return index_1.EJSON.fromJSONValue(internalClone(index_1.EJSON.toJSONValue(val)));
        }
        set.add(val);
        ret = {};
        (0, utils_1.keysOf)(val).forEach(key => {
            if ((0, utils_1.isObjectAndNotNull)(val[key])) {
                if (set.has(val[key]))
                    return;
                set.add(val[key]);
            }
            ret[key] = internalClone(val[key]);
        });
        return ret;
    }
    return internalClone(_val);
};
exports.clone = clone;
//# sourceMappingURL=clone.js.map